<?php

/**
 * @file
 * Contains \Drupal\openai_search\Form\AiSearchForm.
 */

namespace Drupal\openai_search\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

class AiSearchForm extends FormBase {

  /**
   *  {@inheritdoc}
   */
  public function getFormId() {
    return 'ai_search_form';
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Get the configuration object.
    $config = \Drupal::config('openai_search.settings');
    // Get the url from the configuration.
    $restful_url = $config->get('restful_url');
    // Retrieve the X-Secure-Key value from the configuration.
    $X_Secure_Key = $config->get('X-Secure-Key');
    // get the session id from the session.
    $session_id = \Drupal::service('session')->getId();
    // Get the node type from the node object.
    $node = \Drupal::routeMatch()->getParameter('node');
    // Check if the node exists.
    if ($node) {
      $node_type = $node->getType();
      $node_id = $node->id();
      $username = \Drupal::currentUser()->getAccountName();
      $ai_url = $username . "_" . "node_" . $node_id;
      // write to watchdog the node type
      \Drupal::logger('openai_search')->notice("the node type is " . $node_type);
    } else {
      $node_type = '';
      $ai_url = '';
      $node_id = '';
    }

    //
    // write to watchdog the node type
    \Drupal::logger('openai_search')->notice("the node type is " . $node_type);
    $form['#attached']['drupalSettings']['openai_search'] = [
      'restful_url' => $restful_url,
      'xsecurekey' => $X_Secure_Key,
      'use_javascript' => $config->get('use_javascript'),
      'session_id' => $session_id,
      'node_type' => $node_type,
      'ai_url' =>  $ai_url,
    ];
    if ($config->get('use_javascript')) {
      $form['search_history'] = [
        '#markup' => '<div id="search-history-id"></div>',
      ];
    }
    $form['user_search'] = array(
      '#type' => 'textarea',
      '#title' => '',
      '#rows' => 3,
      '#attributes' => [
        'id' => 'user-search-id', // Add your desired ID here
      ],
      '#default_value' => '',
      '#placeholder' => t('Ask me anything...'),
    );
    $form['loading-spinner'] = [
      '#markup' => '<div id="css-wtkai-spinner"><div id="wtk-ai-spinner" class="css-spinner" style="display: none;"></div></div>',
    ];
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Submit'),
      '#attributes' => [
        'id' => 'js-submit', // Adding an ID for easier JavaScript targeting.
        ]
    );
    $form['response'] = [
      '#markup' => '<div id="response"></div>',
    ];
    // Display the response if it exists in the form state.
    if ($form_state->get('response')) {
      $form['response'] = [
        '#markup' => '<br><b></i>' . $form_state->get('user_search') . '<i></b><br> ' . $form_state->get('response'),
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate the form values.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Do something useful.
    // Get the value of the search query.
    $user_search = $form_state->getValue('user_search');
    // Make $user_search url friendly.
    $user_search_encoded = urlencode($user_search);

    // Get the configuration object.
    $config = \Drupal::config('openai_search.settings');
    $session_id = \Drupal::service('session')->getId();
    // Retrieve the 'restful_url' value from the configuration.
    $restful_url = $config->get('restful_url') . "/" . $user_search_encoded . "?UNIQUE_SESSION_COOKIE_NAME=" . $session_id;
    $X_Secure_Key = $config->get('X-Secure-Key');
    // print $restful_url in watchdog
    \Drupal::logger('openai_search')->notice($restful_url);

    // Use curl to send the search query to the RESTful service at $restful_url and get the results.
    // Initialize a cURL session.
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $restful_url); // The URL to fetch.
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true); // Return the transfer as a string.
    curl_setopt($curl, CURLOPT_HTTPHEADER, array('X-Secure-Key: ' . $X_Secure_Key, 'Content-Type: application/json'));
    curl_setopt($curl, CURLOPT_HEADER, false); // Don't include the header in the output.
    curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true); // Follow redirects.

// Execute the cURL session.
    $response = curl_exec($curl);

// Close the cURL session.
    curl_close($curl);

    // Clear the user_search field
    $form_state->setValue('user_search', '');

    // Store the response in the form state.
    $form_state->set('user_search', '');
    $form_state->set('user_search', $user_search);
    $form_state->set('response', $response);


    // Rebuild the form.
    $form_state->setRebuild();

  }

}
